#ifndef _PHONG_H__
#define _PHONG_H__

#include "configh.h"
#include "brdfmodel.h"

#include <cstring>
#include <string.h>

__BEGIN_SBRDFLIB_HEADER

// -------------------------------------------------------------------
// Implementation using Phong model - white color only
class BRDF_Phong: public BRDF
{
protected:
  // The specification of the Phong's model that is white color
  float rho_diff;
  float rho_spec;
  float spec_index;
public:
  BRDF_Phong(float rho_diffuse, float rho_specular, float spec_index);
  virtual ~BRDF_Phong() {}

  // returns cosine corrected value of BRDF given the illumination
  // and viewer angle in RGB, returns false on success
  virtual bool getBRDFValue(int irow, int jcol,
			    float theta_view, float phi_view,
			    float theta_il, float phi_il,
			    float RGB[]) const;
  // returns albedo - value between 0.0 and 1.0
  virtual float getAlbedo(int irow, int jcol, // texture position
			  // viewer direction as two angles in radians
			  float theta_view, float phi_view) const;
  // Make stochastic sampling 
  virtual bool spawnStochasticRay(int irow, int jcol,
				  float rnd1, float rnd2, // input
				  float thetaV, float phiV, // the direction of a viewer
				  float XYZ[3], // output direction
				  float RGB[], // corresponding color
				  float &prob) const; // and probability

  //get probability of sampling for particular direction of the function BRDF.cos(theta),
  //RGB has to be provided as BRDF.cos(theta)
  virtual float getProbOfSampling(int irow, int jcol, // texture position
				  float theta_view, float phiV, // viewer direction as two angles in radians
				  float theta_i, float phi_i, // illumination direction as two angles in radians
				  const float RGB[]) const; // corresponding color
  virtual int getID() const { return EE_Phong;}
  virtual void getString(string &outstr) const;
  virtual void getStringExtended(string &outstr) const;

  // manipulation with BRDF model parameters - connection with possible GUI
  virtual int 
  getNumParameters() const;

  virtual const char* 
  getParameterTitle(int parameterIndex) const;

  virtual const char* 
  getParameterDescription(int parameterIndex) const;

  virtual EParameterUIType
  getParameterSyntax(int parameterIndex) const;

  virtual void 
  setParameter(int parameterIndex, float newvalue[]);

  virtual int
  getParameter(int parameterIndex, float values[]) const;

  virtual int
  getParameterConstraints(int parameterIndex, float minvals[], float maxvals[]) const;
  
};

__END_SBRDFLIB_HEADER

#endif // _PHONG_H__
